@@ -1,4 +1,4 @@
-# Copyright 2013-2020 Reahl Software Services (Pty) Ltd. All rights reserved.
+# Copyright 2013-2022 Reahl Software Services (Pty) Ltd. All rights reserved.
 #
 #    This file is part of Reahl.
 #
@@ -21,6 +21,8 @@ Run 'reahl componentinfo reahl-web-declarative' for configuration information.
 """
 
 import random
+import hmac
+import hashlib
 import urllib.parse
 from datetime import datetime, timedelta
 
@@ -34,6 +36,7 @@ from reahl.component.eggs import ReahlEgg
 from reahl.component.config import Configuration
 from reahl.component.context import ExecutionContext
 from reahl.web.interfaces import UserSessionProtocol, UserInputProtocol, PersistedExceptionProtocol, PersistedFileProtocol
+from reahl.web.csrf import CSRFToken
 from reahl.web.fw import Url
 
 class InvalidKeyException(Exception):
@@ -46,7 +49,9 @@ class UserSession(Base, UserSessionProtocol):
 
     id = Column(Integer, primary_key=True)
     discriminator = Column('row_type', String(40))
-    __mapper_args__ = {'polymorphic_on': discriminator}
+    __mapper_args__ = {
+        'polymorphic_identity':'usersession',
+        'polymorphic_on': discriminator}
 
     idle_lifetime = Column(Integer(), nullable=False, default=0)
     last_activity = Column(DateTime(), nullable=False, default=datetime.now)
@@ -69,13 +74,26 @@ class UserSession(Base, UserSessionProtocol):
     @classmethod
     def initialise_web_session_on(cls, context):
         context.session = cls.get_or_create_session()
-    
+
+    @classmethod
+    def preserve_session(cls, session):
+        Session.expunge(session)
+
+    @classmethod
+    def restore_session(cls, session):
+        Session.add(session)
+
     def __init__(self, **kwargs):
         self.generate_salt()
         self.last_activity = datetime.fromordinal(1)
         self.set_idle_lifetime(False)
         super().__init__(**kwargs)
 
+    def get_csrf_token(self):
+        key = ExecutionContext.get_context().config.web.csrf_key
+        message = self.as_key()
+        return CSRFToken(value=hmac.new(key.encode('utf-8'), msg=message.encode('utf-8'), digestmod=hashlib.sha1).hexdigest())
+
     def is_secured(self):
         context = ExecutionContext.get_context()
         return self.is_within_timeout(context.config.web.idle_secure_lifetime) \
@@ -175,7 +193,9 @@ class SessionData(Base):
 
     id = Column(Integer, primary_key=True)
     discriminator = Column('row_type', String(40))
-    __mapper_args__ = {'polymorphic_on': discriminator}
+    __mapper_args__ = {
+        'polymorphic_identity': 'sessiondata',
+        'polymorphic_on': discriminator}
     
     web_session_id = Column(Integer, ForeignKey('usersession.id', ondelete='CASCADE'), index=True)
     web_session = relationship(UserSession)
